@@ -21,50 +21,16 @@ use strict;
 use vars qw ($VERSION $Version $DEBUG);
 
 use Win32::OLE;
+use Win32::Exchange::Const;
 Win32::OLE->Initialize(Win32::OLE::COINIT_OLEINITIALIZE);
 
-#Win32::OLE->Option('_Unique' => 1);
+Win32::OLE->Option('_Unique' => 1);
 #@ISA = qw(Win32::OLE);
 
 my $Version;
-my $VERSION = $Version = "0.028";
+my $VERSION = $Version = "0.031";
 my $DEBUG = 1;
 
-#CONSTANTS
-
-###Various ADS Objects used in Mailbox Creation ##
-my $ADS_SID_RAW = 0x0;                           #
-my $ADS_SID_HEXSTRING = 0x1;                     #
-my $ADS_SID_SAM = 0x2;                           #
-my $ADS_SID_UPN = 0x3;                           #
-my $ADS_SID_SDDL = 0x4;                          #
-my $ADS_SID_WINNT_PATH = 0x5;                    #
-my $ADS_SID_ACTIVE_DIRECTORY_PATH = 0x6;         #
-my $ADS_SID_SID_BINDING = 0x7;                   #
-my $ADS_RIGHT_EXCH_MODIFY_USER_ATT = 0x02;       # 
-my $ADS_RIGHT_EXCH_MAIL_SEND_AS = 0x08;          #
-my $ADS_RIGHT_EXCH_MAIL_RECEIVE_AS = 0x10;       #
-my $ADS_ACETYPE_ACCESS_ALLOWED = 0x00;           #
-##################################################
-
-####Constants used with OpenDSObject######  NOT USED YET
-my $ADS_SECURE_AUTHENTICATION  = 0x1;    #
-my $ADS_USE_ENCRYPTION         = 0x2;    #
-my $ADS_USE_SSL                = 0x2;    #
-my $ADS_READONLY_SERVER        = 0x4;    #
-my $ADS_PROMPT_CREDENTIALS     = 0x8;    #
-my $ADS_NO_AUTHENTICATION      = 0x10;   #
-my $ADS_FAST_BIND              = 0x20;   #
-my $ADS_USE_SIGNING            = 0x40;   #
-my $ADS_USE_SEALING            = 0x80;   #
-my $ADS_USE_DELEGATION         = 0x100;  #
-my $ADS_SERVER_BIND            = 0x200;  #
-##########################################
-
-###E2K ACE declarations#################
-my $ADS_RIGHT_DS_CREATE_CHILD= 0x1;    #
-my $ADS_ACEFLAG_INHERIT_ACE= 0x2;      #
-########################################
 
 sub new {
   my $server;
@@ -184,10 +150,10 @@ sub GetVersion {
     _ReportArgError("GetVersion",scalar(@_));
     return 0;
   }
-
+ 
   my $serial_val;
   my $serial_version_check_obj = Win32::OLE->new('CDOEXM.ExchangeServer'); #substantiates the possible existance of e2k
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     if ($error_num eq "0x80040154" ||
         $error_num eq "0x800401f3") {
       #0x80040154 Class not registered
@@ -204,11 +170,15 @@ sub GetVersion {
   } else {
     _DebugComment("found e2k tools, so we'll look and see what version of Exchange you have.\n",3);
     if (!_E2kVersionInfo($server_name,$serial_val)) {
-      _DebugComment("Error getting version information from Exchange 2000\n",1);
-      return 0;
+      _DebugComment("Error getting version information from Exchange 2000 tools, let's try the Exch 5.5 way\n",3);
+      if (!_E55VersionInfo($server_name,$serial_val)) {
+        _DebugComment("Error getting version information trying the Exch 5.5 way\n",3);
+        _DebugComment("Error getting version information\n",1);
+        return 0;
+      }
     }
   }
-
+ 
   if ($serial_val =~ /Version (.*) \(Build (.*): Service Pack (.*)\)/i) {
     my %return_struct;
     $return_struct{ver}= $1;
@@ -227,29 +197,49 @@ sub GetVersion {
     return 0;
   }
 }
-
+ 
 sub _E55VersionInfo {
   my $server_name;
   my $error_num;
   my $error_name;
   if (scalar(@_) == 2) {
-    $server_name = $_[0];
+    $server_name = uc($_[0]);
   } else {
     _ReportArgError("_E55VersionInfo",scalar(@_));
     return 0;
   }
   my $serial_val;
-  if (_AdodbExtendedSearch($server_name,"LDAP://RootDSE/configurationNamingContext","(objectCategory=msExchExchangeServer)","name,serialNumber",$serial_val)) {
-    if ($serial_val =~ /Version (.*) \(Build (.*): Service Pack (.*)\)/i) {
-      $_[1] = $serial_val;
-      return 1;
-    } else {
-      _DebugComment("GetVersion failed to produce acceptable results (E55)\n",1);
-      return 0;
-    }
+  my $provider;
+  my $org;
+  my $ou;
+  $provider = Win32::Exchange->new("5.5");
+  if (!$provider) {
+    _DebugComment("new provider create in GetVersion (E55) failed\n",1);
+    return 0;
+  }
+  if ($provider->GetLDAPPath($server_name,$org,$ou)) {
+    _DebugComment("returned -> o=$org,ou=$ou\n",3);
+  } else {
+    _DebugComment("Error Returning from GetLDAPPath in GetVersion (E55)\n",1);
+    return 0;
+  }
+  bless $provider,"Win32::OLE";
+  my $exch_server_obj = $provider->GetObject("","LDAP://$server_name/cn=$server_name,cn=Servers,cn=Configuration,ou=$ou,o=$org");
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
+    _DebugComment("Failed getting the server object in Server container for version info (E55): $error_num,$error_name\n",1);
+    return 0;
+  }
+  $exch_server_obj->GetInfoEx(['serialNumber'],0);
+  $serial_val = $exch_server_obj->{"serialNumber"};
+  if ($serial_val =~ /Version (.*) \(Build (.*): Service Pack (.*)\)/i) {
+    $_[1] = $serial_val;
+    return 1;
+  } else {
+    _DebugComment("GetVersion failed to produce acceptable results (E55)\n",1);
+    return 0;
   }
 }
-
+ 
 sub _E2kVersionInfo {
   my $error_num;
   my $error_name;
@@ -258,20 +248,26 @@ sub _E2kVersionInfo {
   }
   my $server_name = $_[0];
   my $exchange_server = Win32::OLE->new("CDOEXM.ExchangeServer");
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
       _DebugComment("Failed creating object for version information (E2K) on $server_name -> $error_num ($error_name)\n",1);
       return 0;
   }
   $exchange_server->DataSource->Open($server_name);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
+    if ($error_num eq "0x80072032") {
+      #This error might be there if the server is on another domain...  not sure..  I'll need to research more.
+      #It happenned on an E5.5 server anyway so I didn't need the E2K version strucure.
+      _DebugComment("Failed opening object for version information (E2K) on $server_name -> $error_num ($error_name)\n",2);
+    } else {
       _DebugComment("Failed opening object for version information (E2K) on $server_name -> $error_num ($error_name)\n",1);
-      return 0;
+    }
+    return 0;
   }
-
-  #exampe output:
+ 
+  #example output:
   #Version 5.5 (Build 2653.23: Service Pack 4)
   #Version 6.0 (Build 6249.4: Service Pack 3)
-
+ 
   if ($exchange_server->{ExchangeVersion} ne "") {
     $_[1] = $exchange_server->{ExchangeVersion};
     return 1;
@@ -314,7 +310,7 @@ sub AdodbSearch {
   $Conn->Win32::OLE::Open("Active Directory Provider;UID=;PWD=");
   my $path = "<$ldap_path>;$filter;$columns;subtree";
   my $RS = $Conn->Win32::OLE::Execute($path);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("path=$ldap_path\nfilter=$filter\ncolumns=$columns\n",2);
     _DebugComment("Failed Executing ADODB Execute command on $server_name -> $error_num ($error_name)\n",1);
     return 0;
@@ -363,12 +359,12 @@ sub _AdodbExtendedSearch {
   if ($path =~ /^LDAP:\/\/RootDSE\/(.*)/i) {
     $option = $1;
     my $RootDSE = Win32::OLE->GetObject("LDAP://RootDSE");
-    if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+    if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
       _DebugComment("Failed creating object for _AdodbExtendedSearch on $server_name -> $error_num ($error_name)\n",1);
       return 0;
     }
     my $actual_ldap_path = $RootDSE->Get($option);
-    if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+    if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
       _DebugComment("Failed creating object for _AdodbExtendedSearch on $server_name -> $error_num ($error_name)\n",1);
       return 0;
     }
@@ -376,13 +372,13 @@ sub _AdodbExtendedSearch {
   }
   my $string = "<$path>;$filter;$columns;subtree";
   my $Com = Win32::OLE->new("ADODB.Command");
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
       _DebugComment("path=$path\nfilter=$filter\ncolumns=$columns\n",2);
       _DebugComment("Failed creating ADODB.Command object for _AdodbExtendedSearch on $server_name -> $error_num ($error_name)\n",1);
       return 0;
   }
   my $Conn = Win32::OLE->new("ADODB.Connection");
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
       _DebugComment("path=$path\nfilter=$filter\ncolumns=$columns\n",2);
       _DebugComment("Failed creating ADODB.Connection object for version information (E55) on $server_name -> $error_num ($error_name)\n",1);
       return 0;
@@ -391,7 +387,7 @@ sub _AdodbExtendedSearch {
   $Conn->Open("ADs Provider");
   $Com->{ActiveConnection} = $Conn;
   my $RS = $Conn->Execute($string);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
       _DebugComment("path=$path\nfilter=$filter\ncolumns=$columns\n",2);
       _DebugComment("Failed executing ADODB.Command for version information (E55) on $server_name -> $error_num ($error_name)\n",1);
       return 0;
@@ -522,13 +518,13 @@ sub _EnumStorageGroups {
   my $exchange_server = Win32::OLE->new("CDOEXM.ExchangeServer");
 
   $exchange_server->DataSource->Open($server_name);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("Failed opening ADODB ExchangeServer object for Storage Group enumeration on $server_name -> $error_num ($error_name)\n",1);
     return 0;
   }
 
   my @storegroups = Win32::OLE::in($exchange_server->StorageGroups);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("Failed enumerating Storage Groups on $server_name -> $error_num ($error_name)\n",1);
     return 0;
   }
@@ -606,7 +602,7 @@ sub CreateMailbox {
   
   Win32::OLE->LastError(0);
   my $type = Win32::OLE->QueryObjectType($provider);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("failed querying OLE Object for Exchange Server Determination for CreateMailbox ($error_num)\n",1);
     bless $provider,"Win32::Exchange";
     return 0;
@@ -663,7 +659,7 @@ sub _E55CreateMailbox {
   _DebugComment("$recipients_path\n",3);
   bless $ldap_provider,"Win32::OLE";
   my $Recipients = $ldap_provider->GetObject("",$recipients_path);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("Failed opening recipients path on $information_store_server\n",1);
     return 0;
   }
@@ -672,7 +668,7 @@ sub _E55CreateMailbox {
   $Win32::OLE::Warn = 0; #Turn STDERR warnings off because we probably are going to get an error (0x80072030)
 
   $Recipients->GetObject("organizationalPerson", "cn=$mailbox_alias_name");
-  if (!_ErrorCheck("0x80072030",$error_num,$error_name)) {
+  if (!ErrorCheck("0x80072030",$error_num,$error_name)) {
     if ($error_num eq "0x00000000") {
       _DebugComment("$error_num - Mailbox already exists on $information_store_server\n",1);
       $Win32::OLE::Warn=$original_ole_warn_value;
@@ -687,7 +683,7 @@ sub _E55CreateMailbox {
   bless $ldap_provider,"Win32::Exchange";
 
   my $new_mailbox = $Recipients->Create("organizationalPerson", "cn=$mailbox_alias_name");
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error creating Mailbox -> $error_num ($error_name)\n",1);
     $Win32::OLE::Warn=$original_ole_warn_value;
     return 0;
@@ -707,7 +703,7 @@ sub _E55CreateMailbox {
     $new_mailbox->Put($attr => $attrs{$attr}); 
   }
   $new_mailbox->SetInfo;
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error setting attribute on mailbox -> $error_num ($error_name)\n",1);
     $Win32::OLE::Warn=$original_ole_warn_value;
     return 0;
@@ -762,12 +758,12 @@ sub _E2KCreateMailbox {
  
   bless $provider,"Win32::OLE";
   my $user_account = $provider->DataSource->Open("LDAP://$pdc/$user_dist_name");
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("Failed opening NT user account for new mailbox creation on $pdc ($error_num)\n",1);
     return 0;
   }
   my $info_store = $provider->GetInterface( "IMailboxStore");
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("Failed opening mailbox interface on $pdc ($error_num)\n",1);
     if ($error_num eq "0x80004002") {
 
@@ -782,7 +778,7 @@ sub _E2KCreateMailbox {
   }
   _DebugComment("$mailbox_ldap_path\n",3);
   $info_store->CreateMailbox($mailbox_ldap_path);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("Failed creating mailbox for $mailbox_alias_name ($error_num) $error_name\n",1);
     return 0;
   }
@@ -794,7 +790,7 @@ sub _E2KCreateMailbox {
   #i.e. $provider->DataSource->Save(); #may eventually yield an error
   
   $provider->DataSource->Save();
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("Failed saving mailbox for $mailbox_alias_name\n",1);
     return 0;
   }
@@ -852,7 +848,7 @@ sub GetMailbox {
   bless $provider,"Win32::OLE";
   Win32::OLE->LastError(0);
   my $type = Win32::OLE->QueryObjectType($provider);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("failed querying OLE Object for Exchange Server Determination for CreateMailbox\n",1);
     bless $provider,"Win32::Exchange";
     return 0;
@@ -908,7 +904,7 @@ sub _E55GetMailbox {
   my $recipients_path = "LDAP://$information_store_server/cn=Recipients,ou=$ou,o=$org";
   bless $ldap_provider,"Win32::OLE";
   my $Recipients = $ldap_provider->GetObject("",$recipients_path);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("Failed opening recipients path on $information_store_server\n",1);
     return 0;
   }
@@ -917,7 +913,7 @@ sub _E55GetMailbox {
   $Win32::OLE::Warn = 0; #Turn STDERR warnings off because we probably are going to get an error (0x80072030)
 
   my $mailbox = $Recipients->GetObject("organizationalPerson", "cn=$mailbox_alias_name");
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("Unable to Get the mailbox object for $mailbox_alias_name on $information_store_server ($error_num)\n",1);
     $Win32::OLE::Warn=$original_ole_warn_value;
     return 0;
@@ -951,13 +947,13 @@ sub _E2KGetMailbox {
 
   bless $provider,"Win32::OLE";
   $provider->DataSource->Open("LDAP://$dc/$user_dist_name");
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("Failed opening AD user account for mailbox retrieval on $dc ($error_num)\n",1);
     return 0;
   }
   my $user_obj_path = $provider->DataSource->{SourceURL};
   my $user_obj = Win32::OLE->GetObject($user_obj_path);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("Failed opening SourceURL for GetMailbox ($error_num)\n",1);
     return 0;
   }
@@ -979,7 +975,7 @@ sub SetAttributes {
   bless $provider,"Win32::OLE";
   Win32::OLE->LastError(0);
   my $type = Win32::OLE->QueryObjectType($provider);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("failed querying OLE Object type for Exchange Server Determination during call to SetAttributes\n",1);
     bless $provider,"Win32::Exchange";
     return 0;
@@ -1027,7 +1023,7 @@ sub _E55SetAttributes {
     $mailbox->Put($attr => $attrs{$attr}); 
   }
   $mailbox->SetInfo;
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error setting attribute on mailbox -> $error_num ($error_name)\n",1);
     $Win32::OLE::Warn=$original_ole_warn_value;
     return 0;
@@ -1057,7 +1053,7 @@ sub _E2KSetAttributes {
   bless $user_account,"Win32::OLE";
   foreach my $interface (keys %attrs) {
     my $mailbox_interface = $user_account->GetInterface($interface);
-    if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+    if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
       _DebugComment("error getting mailbox interface -> $error_num ($error_name)\n",1);
       return 0;
     }
@@ -1066,7 +1062,7 @@ sub _E2KSetAttributes {
     }
     $user_account->DataSource->Save();
   }
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error setting attribute on mailbox -> $error_num ($error_name)\n",1);
     return 0;
   }
@@ -1090,7 +1086,7 @@ sub GetOwner {
   bless $provider,"Win32::OLE";
   Win32::OLE->LastError(0);
   my $type = Win32::OLE->QueryObjectType($provider);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("failed querying OLE Object type for GetOwner\n",1);
     bless $provider,"Win32::Exchange";
     return 0;
@@ -1122,7 +1118,7 @@ sub _E55GetOwner {
   if (scalar(@_) > 1) {
     $mailbox = $_[0];
     if (scalar(@_) == 2) {
-      $returned_sid_type = $ADS_SID_WINNT_PATH;    
+      $returned_sid_type = ADS_SID_WINNT_PATH;    
     } elsif (scalar(@_) == 3) {
       $returned_sid_type = $_[2];    
     } else {
@@ -1138,25 +1134,25 @@ sub _E55GetOwner {
   bless $mailbox,"Win32::OLE";
 
   my $sid = Win32::OLE->new("ADsSID");
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error creating ADsSID object -> $error_num ($error_name)\n",1);
     return 0;
   }
   $mailbox->GetInfoEx(["Assoc-NT-Account"],0);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error populating the property cache for Assoc-NT-Account -> $error_num ($error_name)\n",1);
     return 0;
   }
 
-  $sid->SetAs($ADS_SID_HEXSTRING,$mailbox->{'Assoc-NT-Account'});
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  $sid->SetAs(ADS_SID_HEXSTRING,$mailbox->{'Assoc-NT-Account'});
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error creating ADsSID object -> $error_num ($error_name)\n",1);
     return 0;
   }
 
   my $siduser = $sid->GetAs($returned_sid_type);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
-    if (_ErrorCheck("0x80070534",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
+    if (ErrorCheck("0x80070534",$error_num,$error_name)) {
       _DebugComment("there was an error validating the SID from the Domain Controller (the account doesn't seem to exist anymore) -> $error_num ($error_name)\n",1);
       return 0;
     }
@@ -1188,18 +1184,18 @@ sub SetOwner {
   }
 
   my $sid = Win32::OLE->new("ADsSID");
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error creating security object (ADsSID) -> $error_num ($error_name)\n",1);
     return 0;
   }
-  $sid->SetAs($ADS_SID_WINNT_PATH, "WinNT://$dc/$username,user");
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  $sid->SetAs(ADS_SID_WINNT_PATH, "WinNT://$dc/$username,user");
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error setting security object at an ADS_SID_WINNT_PATH -> $error_num ($error_name)\n",1);
     return 0;
   }
 
-  my $sidHex = $sid->GetAs($ADS_SID_HEXSTRING);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  my $sidHex = $sid->GetAs(ADS_SID_HEXSTRING);
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error converting security object at an ADS_SID_HEXSTRING -> $error_num ($error_name)\n",1);
     return 0;
   }
@@ -1208,7 +1204,7 @@ sub SetOwner {
   $new_mailbox->Put("Assoc-NT-Account", $sidHex );
   $new_mailbox->SetInfo;
 
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error setting owner information on mailbox -> $error_num ($error_name)\n",1);
     bless $new_mailbox,"Win32::Exchange";
     return 0;      
@@ -1217,6 +1213,46 @@ sub SetOwner {
   return 1;
 }
 
+sub _E55GetPerms {
+  #Need to work on this.
+  if (scalar(@_) != 2) {
+    _ReportArgError("GetPerms [5.5]",scalar(@_));
+    return 0;
+  }
+  if (ref($_[1]) ne "ARRAY") {
+    _DebugComment("permissions list must be an array reference (e55)\n",1);
+    return 0;
+  }
+  my $mailbox = $_[0];
+
+  my $sec = Win32::OLE->CreateObject("ADsSecurity");
+  my $error_num;
+  my $error_name;
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
+    _DebugComment("error creating security object (ADSSecurity) -> $error_num ($error_name)\n",1);
+    if ($error_num eq "0x80004002") {
+      _DebugComment("Error:  No such interface supported.\n  Note:  Make sure you have the ADSSecurity.DLL from the ADSI SDK regisered on this system\n",2);
+    }
+    return 0;
+  }
+
+  bless $mailbox,"Win32::OLE";
+  
+  my $sd = $sec->GetSecurityDescriptor($mailbox->{ADsPath});
+  
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
+    _DebugComment("error querying security descriptor for mailbox -> $error_num ($error_name)\n",1);
+    return 0;
+  }
+  my $dacl = $sd->{DiscretionaryAcl};
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
+    _DebugComment("error querying discretionary acl for mailbox -> $error_num ($error_name)\n",1);
+    return 0;
+  }
+  @{$_[1]} = Win32::OLE::in($dacl);
+  return 1;
+}
+
 sub SetPerms {
   my $error_num;
   my $error_name;
@@ -1225,7 +1261,7 @@ sub SetPerms {
   bless $provider,"Win32::OLE";
   Win32::OLE->LastError(0);
   my $type = Win32::OLE->QueryObjectType($provider);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("failed querying OLE Object type for Exchange Server Determination during call to SetAttributes\n",1);
     bless $provider,"Win32::Exchange";
     return 0;
@@ -1265,7 +1301,7 @@ sub _E55SetPerms {
   my $sec = Win32::OLE->CreateObject("ADsSecurity");
   my $error_num;
   my $error_name;
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error creating security object (ADSSecurity) -> $error_num ($error_name)\n",1);
     if ($error_num eq "0x80004002") {
       _DebugComment("Error:  No such interface supported.\n  Note:  Make sure you have the ADSSecurity.DLL from the ADSI SDK regisered on this system\n",2);
@@ -1277,12 +1313,12 @@ sub _E55SetPerms {
   
   my $sd = $sec->GetSecurityDescriptor($new_mailbox->{ADsPath});
   
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error querying security descriptor for mailbox -> $error_num ($error_name)\n",1);
     return 0;
   }
   my $dacl = $sd->{DiscretionaryAcl};
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error querying discretionary acl for mailbox -> $error_num ($error_name)\n",1);
     return 0;
   }
@@ -1290,19 +1326,19 @@ sub _E55SetPerms {
   foreach my $userid (@perms_list) {
     _DebugComment("      -Setting perms for $userid\n",3);
     my $ace = Win32::OLE->CreateObject("AccessControlEntry");
-    if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+    if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
       _DebugComment("error creating access control entry for mailbox -> $error_num ($error_name)\n",1);
       return 0;
     }
 
     my %properties;
     $properties{Trustee}=$userid;
-    $properties{AccessMask}=$ADS_RIGHT_EXCH_MODIFY_USER_ATT | $ADS_RIGHT_EXCH_MAIL_SEND_AS | $ADS_RIGHT_EXCH_MAIL_RECEIVE_AS;
-    $properties{AceType}=$ADS_ACETYPE_ACCESS_ALLOWED;
+    $properties{AccessMask}=ADS_RIGHT_EXCH_MODIFY_USER_ATT | ADS_RIGHT_EXCH_MAIL_SEND_AS | ADS_RIGHT_EXCH_MAIL_RECEIVE_AS;
+    $properties{AceType}=ADS_ACETYPE_ACCESS_ALLOWED;
 
     foreach my $property (keys %properties) {
       $ace->LetProperty($property,$properties{$property}); 
-      if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+      if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
         _DebugComment("error setting $property for mailbox -> $error_num ($error_name)\n",1);
         return 0;
       }
@@ -1310,23 +1346,23 @@ sub _E55SetPerms {
 
 
     $dacl->AddAce($ace);
-    if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+    if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
       _DebugComment("error adding access control entry to perms list -> $error_num ($error_name)\n",1);
       return 0;
     }
   }
   $sd->LetProperty("DiscretionaryAcl",$dacl); 
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error setting  discretionary acl on security security descriptor -> $error_num ($error_name)\n",1);
     return 0;
   }
   $sec->SetSecurityDescriptor($sd);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error setting security descriptor on security object -> $error_num ($error_name)\n",1);
     return 0;
   }
   $new_mailbox->SetInfo;
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error setting permissions on mailbox -> $error_num ($error_name)\n",1);
     return 0;
   }
@@ -1352,14 +1388,14 @@ sub _E2KSetPerms {
 
   my $ldap_user_path = $cdo_user_obj->{DataSource}->{SourceURL};
   my $ldap_user_obj = Win32::OLE->GetObject($ldap_user_path);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("Error querying Source URL for CDO.Person object ($error_num)\n",1);
     return 0;
   }
 
   #http://support.microsoft.com/default.aspx?scid=KB;EN-US;Q310866
   my $sd = $ldap_user_obj->{'MailboxRights'};
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("Error querying MailboxRights property ($error_num)\n",1);
     _DebugComment("- make sure you are using Exchange 2000 SP1+hotfix or higher [server & client]\n",2);
     _DebugComment('  http://support.microsoft.com/default.aspx?scid=KB;EN-US;Q302926'."\n",3);
@@ -1367,7 +1403,7 @@ sub _E2KSetPerms {
   }
 
   my $dacl = $sd->{DiscretionaryAcl};
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("Error getting DiscretionaryAcl ($error_num)\n",1);
     return 0;
   }
@@ -1384,14 +1420,14 @@ sub _E2KSetPerms {
       return 0;
     }
     my $Ace = Win32::OLE->new("AccessControlEntry");
-    if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+    if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
       _DebugComment("Error creating new ACE ($error_num)\n",1);
       return 0;
     }
     my %properties;
-    $properties{AccessMask}=$ADS_RIGHT_DS_CREATE_CHILD;
-    $properties{AceType}=$ADS_ACETYPE_ACCESS_ALLOWED;
-    $properties{AceFlags}=$ADS_ACEFLAG_INHERIT_ACE;
+    $properties{AccessMask}=ADS_RIGHT_DS_CREATE_CHILD;
+    $properties{AceType}=ADS_ACETYPE_ACCESS_ALLOWED;
+    $properties{AceFlags}=ADS_ACEFLAG_INHERIT_ACE;
     $properties{Flags}=0;
     $properties{Trustee}=$user_account;
     $properties{ObjectType}=0;
@@ -1402,29 +1438,29 @@ sub _E2KSetPerms {
       }
   
       $Ace->LetProperty($property,$properties{$property});
-      if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+      if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
         _DebugComment("Error setting $property ($error_num)\n",1);
         return 0;
       }
     }
     $dacl->AddAce($Ace);
-    if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+    if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
       _DebugComment("Error adding AccessControlEntry to AccessControlList: ($error_num)\n",1);
       return 0;
     }
   }
   $sd->LetProperty('DiscretionaryAcl',$dacl);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("Error setting AccessControlList to Security Descriptor: ($error_num)\n",1);
     return 0;
   }
   $ldap_user_obj->LetProperty('MailboxRights',$sd);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("Error modfying Mailbox Security entry: ($error_num)\n",1);
     return 0;
   }
   $ldap_user_obj->SetInfo();
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("Error setting information to Mailbox Security entry: ($error_num)\n",1);
     return 0;
   }
@@ -1438,7 +1474,7 @@ sub AddDLMembers {
   bless $provider,"Win32::OLE";
   Win32::OLE->LastError(0);
   my $type = Win32::OLE->QueryObjectType($provider);
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("failed querying OLE Object type for Exchange Server Determination during call to SetAttributes\n",1);
     bless $provider,"Win32::Exchange";
     return 0;
@@ -1505,7 +1541,7 @@ sub _E55AddDLMembers {
   my $exch_dl = $ldap_provider->GetObject("","LDAP://$server_name/cn=$exch_dl_name,cn=Distribution Lists,ou=$ou,o=$org");
   my $error_num;
   my $error_name;
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error querying distribution list ($exch_dl_name) -> $error_num ($error_name)\n",1);
     return 0;
   }
@@ -1540,7 +1576,7 @@ sub _E55AddDLMembers {
   }
   $exch_dl->Put('member', $exch_members);
   $exch_dl->SetInfo;
-  if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+  if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
     _DebugComment("error setting new member for distribution list ($exch_dl_name) -> $error_num ($error_name)\n",1);
     return 0;
   }
@@ -1576,29 +1612,29 @@ sub _E2KAddDLMembers {
       return 0;
     }
     my $RootDSE = Win32::OLE->GetObject("LDAP://RootDSE");
-    if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+    if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
       _DebugComment("Failed Getting RootDSE ($error_num)\n",1);
       return 0;
     }
     my $dc = $RootDSE->Get("dnsHostName");
-    if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+    if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
       _DebugComment("Error getting RootDSE dns host name ($error_num)\n",1);
       return 0;
     }
     my @dc_array = split(/\./,$dc);
     my $ldap_obj = Win32::OLE->new("ADsNamespaces");
-    if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+    if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
       _DebugComment("Error creating new ADsNamespaces object ($error_num)\n",1);
       return 0;
     }
     my $group_obj = $ldap_obj->GetObject("","LDAP://$dc_array[0]/$group_dn");
-    if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+    if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
       _DebugComment("Error opening distribution list on $dc_array[0] ($error_num)\n",1);
       return 0;
     }
   
     $group_obj->Add("LDAP://$dc_array[0]/$user_dn");
-    if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+    if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
       if ($error_num eq "0x80071392") {
         _DebugComment("Error adding user ($username) to distribution list [they are already a member]\n",1);
       } else {
@@ -1608,7 +1644,7 @@ sub _E2KAddDLMembers {
     }
  
     $group_obj->SetInfo;
-    if (!_ErrorCheck("0x00000000",$error_num,$error_name)) {
+    if (!ErrorCheck("0x00000000",$error_num,$error_name)) {
       _DebugComment("Error committing addition to distribution list ($error_num)\n",1);
       return 0;
     }
@@ -1616,7 +1652,7 @@ sub _E2KAddDLMembers {
   return 1;
 }
 
-sub _ErrorCheck {
+sub ErrorCheck {
   my $last_error_expected = $_[0];
   my $error_num;
   my $error_name;
@@ -1645,4 +1681,5 @@ sub _DebugComment {
   print "$_[0]" if ($DEBUG > ($_[1] - 1));
   return 1;
 }
+
 1;
\ No newline at end of file
